@@ -1,6 +1,8 @@
 # Copyright (c) Microsoft Corporation. All rights reserved.
 # Licensed under the MIT License. See License.txt in the project root for license information.
 
+from typing import Optional
+
 from cdm.persistence.cdmfolder.types.projections.operation_base import OperationBase
 
 
@@ -10,4 +12,4 @@ class OperationAddAttributeGroup(OperationBase):
     def __init__(self):
         super().__init__()
 
-        # TODO (sukanyas): Property to be defined
+        self.attributeGroupName = None  # type: Optional[str]
