@@ -36,4 +36,23 @@ class StringUtils:
 
     @staticmethod
     def is_null_or_white_space(s: str) -> bool:
-        return s is None or s.strip() is ''
+        return s is None or s.strip() == ''
+
+    @staticmethod
+    def _replace(source: str, pattern: str, value: str) -> str:
+        if len(pattern) > 1:
+            # This exception is just for safety since Python doesn't support char type.
+            raise Exception('Pattern should have size 1.')
+
+        if value is None:
+            value = ''
+
+        lower_case_pattern = pattern.lower()
+        upper_case_pattern = pattern.upper()
+        upper_case_value = ''
+        
+        if value:
+            upper_case_value = value[0].upper() + value[1:]
+
+        result = source.replace('{{{}}}'.format(lower_case_pattern), value)
+        return result.replace('{{{}}}'.format(upper_case_pattern), upper_case_value)
\ No newline at end of file
