@@ -5,7 +5,8 @@ import os
 import unittest
 
 from tests.cdm.projection.attribute_context_util import AttributeContextUtil
-from tests.common import async_test, TestHelper, TestUtils
+from tests.common import async_test, TestHelper
+from tests.utilities.projection_test_utils import ProjectionTestUtils
 
 
 class ProjectionAttributeContextTest(unittest.TestCase):
@@ -27,9 +28,9 @@ class ProjectionAttributeContextTest(unittest.TestCase):
 
         ent_test_entity_string_reference = await corpus.fetch_object_async('local:/{}.cdm.json/{}'.format(entity_name, entity_name), manifest)
         self.assertIsNotNone(ent_test_entity_string_reference)
-        resolved_test_entity_string_reference = await TestUtils._get_resolved_entity(corpus, ent_test_entity_string_reference, [])
+        resolved_test_entity_string_reference = await ProjectionTestUtils.get_resolved_entity(corpus, ent_test_entity_string_reference, [])
         self.assertIsNotNone(resolved_test_entity_string_reference)
-        AttributeContextUtil.validate_attribute_context(self, corpus, expected_output_path, entity_name, resolved_test_entity_string_reference)
+        await AttributeContextUtil.validate_attribute_context(self, corpus, expected_output_path, entity_name, resolved_test_entity_string_reference)
 
     @async_test
     async def test_entity_entity_reference(self):
@@ -43,9 +44,9 @@ class ProjectionAttributeContextTest(unittest.TestCase):
 
         ent_test_entity_entity_reference = await corpus.fetch_object_async('local:/{}.cdm.json/{}'.format(entity_name, entity_name), manifest)
         self.assertIsNotNone(ent_test_entity_entity_reference)
-        resolved_test_entity_entity_reference = await TestUtils._get_resolved_entity(corpus, ent_test_entity_entity_reference, [])
+        resolved_test_entity_entity_reference = await ProjectionTestUtils.get_resolved_entity(corpus, ent_test_entity_entity_reference, [])
         self.assertIsNotNone(resolved_test_entity_entity_reference)
-        AttributeContextUtil.validate_attribute_context(self, corpus, expected_output_path, entity_name, resolved_test_entity_entity_reference)
+        await AttributeContextUtil.validate_attribute_context(self, corpus, expected_output_path, entity_name, resolved_test_entity_entity_reference)
 
     @async_test
     async def test_entity_projection(self):
@@ -59,9 +60,9 @@ class ProjectionAttributeContextTest(unittest.TestCase):
 
         ent_test_entity_projection = await corpus.fetch_object_async('local:/{}.cdm.json/{}'.format(entity_name, entity_name), manifest)
         self.assertIsNotNone(ent_test_entity_projection)
-        resolved_test_entity_projection = await TestUtils._get_resolved_entity(corpus, ent_test_entity_projection, [])
+        resolved_test_entity_projection = await ProjectionTestUtils.get_resolved_entity(corpus, ent_test_entity_projection, [])
         self.assertIsNotNone(resolved_test_entity_projection)
-        AttributeContextUtil.validate_attribute_context(self, corpus, expected_output_path, entity_name, resolved_test_entity_projection)
+        await AttributeContextUtil.validate_attribute_context(self, corpus, expected_output_path, entity_name, resolved_test_entity_projection)
 
     @async_test
     async def test_entity_nested_projection(self):
@@ -75,9 +76,9 @@ class ProjectionAttributeContextTest(unittest.TestCase):
 
         ent_test_entity_nested_projection = await corpus.fetch_object_async('local:/{}.cdm.json/{}'.format(entity_name, entity_name), manifest)
         self.assertIsNotNone(ent_test_entity_nested_projection)
-        resolved_test_entity_nested_projection = await TestUtils._get_resolved_entity(corpus, ent_test_entity_nested_projection, [])
+        resolved_test_entity_nested_projection = await ProjectionTestUtils.get_resolved_entity(corpus, ent_test_entity_nested_projection, [])
         self.assertIsNotNone(resolved_test_entity_nested_projection)
-        AttributeContextUtil.validate_attribute_context(self, corpus, expected_output_path, entity_name, resolved_test_entity_nested_projection)
+        await AttributeContextUtil.validate_attribute_context(self, corpus, expected_output_path, entity_name, resolved_test_entity_nested_projection)
 
     @async_test
     async def test_entity_attribute_string_reference(self):
@@ -91,9 +92,9 @@ class ProjectionAttributeContextTest(unittest.TestCase):
 
         ent_test_entity_attribute_string_reference = await corpus.fetch_object_async('local:/{}.cdm.json/{}'.format(entity_name, entity_name), manifest)
         self.assertIsNotNone(ent_test_entity_attribute_string_reference)
-        resolved_test_entity_attribute_string_reference = await TestUtils._get_resolved_entity(corpus, ent_test_entity_attribute_string_reference, [])
+        resolved_test_entity_attribute_string_reference = await ProjectionTestUtils.get_resolved_entity(corpus, ent_test_entity_attribute_string_reference, [])
         self.assertIsNotNone(resolved_test_entity_attribute_string_reference)
-        AttributeContextUtil.validate_attribute_context(self, corpus, expected_output_path, entity_name, resolved_test_entity_attribute_string_reference)
+        await AttributeContextUtil.validate_attribute_context(self, corpus, expected_output_path, entity_name, resolved_test_entity_attribute_string_reference)
 
     @async_test
     async def test_entity_attribute_entity_reference(self):
@@ -107,9 +108,9 @@ class ProjectionAttributeContextTest(unittest.TestCase):
 
         ent_test_entity_attribute_entity_reference = await corpus.fetch_object_async('local:/{}.cdm.json/{}'.format(entity_name, entity_name), manifest)
         self.assertIsNotNone(ent_test_entity_attribute_entity_reference)
-        resolved_test_entity_attribute_entity_reference = await TestUtils._get_resolved_entity(corpus, ent_test_entity_attribute_entity_reference, [])
+        resolved_test_entity_attribute_entity_reference = await ProjectionTestUtils.get_resolved_entity(corpus, ent_test_entity_attribute_entity_reference, [])
         self.assertIsNotNone(resolved_test_entity_attribute_entity_reference)
-        AttributeContextUtil.validate_attribute_context(self, corpus, expected_output_path, entity_name, resolved_test_entity_attribute_entity_reference)
+        await AttributeContextUtil.validate_attribute_context(self, corpus, expected_output_path, entity_name, resolved_test_entity_attribute_entity_reference)
 
     @async_test
     async def test_entity_attribute_projection(self):
@@ -123,9 +124,9 @@ class ProjectionAttributeContextTest(unittest.TestCase):
 
         ent_test_entity_attribute_projection = await corpus.fetch_object_async('local:/{}.cdm.json/{}'.format(entity_name, entity_name), manifest)
         self.assertIsNotNone(ent_test_entity_attribute_projection)
-        resolved_test_entity_attribute_projection = await TestUtils._get_resolved_entity(corpus, ent_test_entity_attribute_projection, [])
+        resolved_test_entity_attribute_projection = await ProjectionTestUtils.get_resolved_entity(corpus, ent_test_entity_attribute_projection, [])
         self.assertIsNotNone(resolved_test_entity_attribute_projection)
-        AttributeContextUtil.validate_attribute_context(self, corpus, expected_output_path, entity_name, resolved_test_entity_attribute_projection)
+        await AttributeContextUtil.validate_attribute_context(self, corpus, expected_output_path, entity_name, resolved_test_entity_attribute_projection)
 
     @async_test
     async def test_entity_attribute_nested_projection(self):
@@ -139,9 +140,9 @@ class ProjectionAttributeContextTest(unittest.TestCase):
 
         ent_test_entity_attribute_nested_projection = await corpus.fetch_object_async('local:/{}.cdm.json/{}'.format(entity_name, entity_name), manifest)
         self.assertIsNotNone(ent_test_entity_attribute_nested_projection)
-        resolved_test_entity_attribute_nested_projection = await TestUtils._get_resolved_entity(corpus, ent_test_entity_attribute_nested_projection, [])
+        resolved_test_entity_attribute_nested_projection = await ProjectionTestUtils.get_resolved_entity(corpus, ent_test_entity_attribute_nested_projection, [])
         self.assertIsNotNone(resolved_test_entity_attribute_nested_projection)
-        AttributeContextUtil.validate_attribute_context(self, corpus, expected_output_path, entity_name, resolved_test_entity_attribute_nested_projection)
+        await AttributeContextUtil.validate_attribute_context(self, corpus, expected_output_path, entity_name, resolved_test_entity_attribute_nested_projection)
 
     @async_test
     async def test_entity_trait(self):
@@ -155,9 +156,9 @@ class ProjectionAttributeContextTest(unittest.TestCase):
 
         ent_test_entity_trait = await corpus.fetch_object_async('local:/{}.cdm.json/{}'.format(entity_name, entity_name), manifest)
         self.assertIsNotNone(ent_test_entity_trait)
-        resolved_test_entity_trait = await TestUtils._get_resolved_entity(corpus, ent_test_entity_trait, [])
+        resolved_test_entity_trait = await ProjectionTestUtils.get_resolved_entity(corpus, ent_test_entity_trait, [])
         self.assertIsNotNone(resolved_test_entity_trait)
-        AttributeContextUtil.validate_attribute_context(self, corpus, expected_output_path, entity_name, resolved_test_entity_trait)
+        await AttributeContextUtil.validate_attribute_context(self, corpus, expected_output_path, entity_name, resolved_test_entity_trait)
 
         # Attribute Name
         self.assertEqual('TestAttribute', resolved_test_entity_trait.attributes[0].name)
@@ -182,9 +183,9 @@ class ProjectionAttributeContextTest(unittest.TestCase):
 
         ent_test_entity_extends_trait = await corpus.fetch_object_async('local:/{}.cdm.json/{}'.format(entity_name, entity_name), manifest)
         self.assertIsNotNone(ent_test_entity_extends_trait)
-        resolved_test_entity_extends_trait = await TestUtils._get_resolved_entity(corpus, ent_test_entity_extends_trait, [])
+        resolved_test_entity_extends_trait = await ProjectionTestUtils.get_resolved_entity(corpus, ent_test_entity_extends_trait, [])
         self.assertIsNotNone(resolved_test_entity_extends_trait)
-        AttributeContextUtil.validate_attribute_context(self, corpus, expected_output_path, entity_name, resolved_test_entity_extends_trait)
+        await AttributeContextUtil.validate_attribute_context(self, corpus, expected_output_path, entity_name, resolved_test_entity_extends_trait)
 
         # Attribute Name
         self.assertEqual('TestExtendsTraitAttribute', resolved_test_entity_extends_trait.attributes[0].name)
@@ -209,9 +210,9 @@ class ProjectionAttributeContextTest(unittest.TestCase):
 
         ent_test_projection_trait = await corpus.fetch_object_async('local:/{}.cdm.json/{}'.format(entity_name, entity_name), manifest)
         self.assertIsNotNone(ent_test_projection_trait)
-        resolved_test_projection_trait = await TestUtils._get_resolved_entity(corpus, ent_test_projection_trait, [])
+        resolved_test_projection_trait = await ProjectionTestUtils.get_resolved_entity(corpus, ent_test_projection_trait, [])
         self.assertIsNotNone(resolved_test_projection_trait)
-        AttributeContextUtil.validate_attribute_context(self, corpus, expected_output_path, entity_name, resolved_test_projection_trait)
+        await AttributeContextUtil.validate_attribute_context(self, corpus, expected_output_path, entity_name, resolved_test_projection_trait)
 
         # Attribute Name
         self.assertEqual('TestProjectionAttribute', resolved_test_projection_trait.attributes[0].name)
@@ -236,9 +237,9 @@ class ProjectionAttributeContextTest(unittest.TestCase):
 
         ent_test_projection_extends_trait = await corpus.fetch_object_async('local:/{}.cdm.json/{}'.format(entity_name, entity_name), manifest)
         self.assertIsNotNone(ent_test_projection_extends_trait)
-        resolved_test_projection_extends_trait = await TestUtils._get_resolved_entity(corpus, ent_test_projection_extends_trait, [])
+        resolved_test_projection_extends_trait = await ProjectionTestUtils.get_resolved_entity(corpus, ent_test_projection_extends_trait, [])
         self.assertIsNotNone(resolved_test_projection_extends_trait)
-        AttributeContextUtil.validate_attribute_context(self, corpus, expected_output_path, entity_name, resolved_test_projection_extends_trait)
+        await AttributeContextUtil.validate_attribute_context(self, corpus, expected_output_path, entity_name, resolved_test_projection_extends_trait)
 
         # Attribute Name
         self.assertEqual('TestProjectionAttribute', resolved_test_projection_extends_trait.attributes[0].name)
